using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using TelnetExpect;
using System.Diagnostics;

namespace TelnetTests
{
    [TestFixture]
    public class TnapTests :TestsBase
    {
        const byte NTLM_ACCEPT = 0x03;
        const byte NTLM_REJECT = 0x04;
        //
        byte[] empty = new byte[] { };
        //
        bool m_doMunge;

        protected override byte[] MungeResponseBuffer(byte[] expectedResponses, System.IO.MemoryStream actualMs)
        {
            if (!m_doMunge) {
                return expectedResponses;
            }
            try {
                byte[] expRsp2 = (byte[])expectedResponses.Clone();
                const int AuthMsgOffset = 109;
                const int PayloadOffset = 22 * 4 + AuthMsgOffset;
                const int PayloadLength = 24+24+16;
                    //new int[] {PayloadOffset, 24}, // LmChlgResp        // 197
                    //new int[] {24 + PayloadOffset, 24}, // NtChlgResp   // 221
                    //new int[] {24 + 24 + PayloadOffset, 16}, // foo
                byte[] actual = actualMs.ToArray();
                byte follow0 = expectedResponses[0 + PayloadOffset + PayloadLength];
                byte follow1 = expectedResponses[1 + PayloadOffset + PayloadLength];
                Assert.AreEqual(IAC, follow0, "IAC==follow0");
                Assert.AreEqual(SE, follow1, "SE==follow1");
                Array.Copy(actual, PayloadOffset, expRsp2, PayloadOffset, PayloadLength);
                //
                byte[] tmpR = new byte[PayloadLength];
                Array.Copy(actual, PayloadOffset, tmpR, 0, PayloadLength);
                byte[] tmp255 = Escape255s(tmpR, 0, PayloadLength);
                if (tmp255.Length != tmpR.Length) {
                    System.Diagnostics.Trace.WriteLine("255sssssssssssssssssssssssssss");
                } else {
                    System.Diagnostics.Trace.WriteLine("(no 255s)");
                }
                return expRsp2;
            } finally {
                m_doMunge = false;
            }
        }

        private static byte[] Escape255s(byte[] buffer, int offset, int count)
        {
            if (-1 == Array.IndexOf<byte>(buffer, 255, offset, count)) {
                // Not 255s needing encoding.  But at wrong offset etc?
                if (offset != 0 || count != buffer.Length) {
                    Debug.Fail("This state not tested.");
                    byte[] abs = new byte[count];
                    Array.Copy(buffer, offset, abs, 0, count);
                    return abs;
                } else {
                    return buffer;
                }
            }
            //else
            List<byte> list = new List<byte>(count + 100);
            int newCount = count;
            for (int i = 0; i < count; ++i) {
                byte cur = buffer[offset + i];
                if (cur == 255) {
                    list.Add(255);
                    ++newCount;
                }
                list.Add(cur);
            }
            byte[] tmp = list.ToArray();
            Debug.Assert(tmp.Length == newCount, "lengths!");
            return tmp;
        }

        //----
        public TnapTests()
        {
            TestsBase.EnsureConsoleTraceListener();
        }

        protected override TelnetStream Create_Testee(System.IO.Stream conn)
        {
            TelnetStream dut = new TelnetStream(conn);
            const bool ForceEnabled = true;
            dut.AddOptionHandler(new FailPromptAuthenticationNtlmTelnetHandler(ForceEnabled));
            return dut;
        }

        byte[] GetTnapServerDoThenChallenge(byte[] followedBy)
        {
            byte[] peerPackets = {
                IAC, DO, OptionCode.AuthenticationOption,
                //----
                IAC, SB, OptionCode.AuthenticationOption, 1,
                    0x0F, 0x00,
                    IAC, SE,
                //---- Challenge ----
                0xFF, 0xFA, 0x25, 0x02, 0x0F, 0x00, 0x01, 0x9E,  0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00,     // %....? .......
                //
                0x4E, 0x54, 0x4C, 0x4D, 0x53, 0x53, 0x50, 0x00,  0x02, 0x00, 0x00, 0x00, 0x0E, 0x00, 0x0E, 0x00,  // NTLMSSP. ........
                0x38, 0x00, 0x00, 0x00, 0x15, 0x82, 0x8A, 0xE2,  0x0F, 0x42, 0x81, 0xE5, 0xDE, 0xE1, 0xBB, 0x88,  // 8....?? .B?
                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x58, 0x00, 0x58, 0x00, 0x46, 0x00, 0x00, 0x00,  // ........ X.X.F...
                0x05, 0x01, 0x28, 0x0A, 0x00, 0x00, 0x00, 0x0F,  0x41, 0x00, 0x4C, 0x00, 0x41, 0x00, 0x4E, 0x00,  // ..(..... A.L.A.N.
                0x50, 0x00, 0x43, 0x00, 0x31, 0x00, 0x02, 0x00,  0x0E, 0x00, 0x41, 0x00, 0x4C, 0x00, 0x41, 0x00,  // P.C.1... ..A.L.A.
                0x4E, 0x00, 0x50, 0x00, 0x43, 0x00, 0x31, 0x00,  0x01, 0x00, 0x0E, 0x00, 0x41, 0x00, 0x4C, 0x00,  // N.P.C.1. ....A.L.
                0x41, 0x00, 0x4E, 0x00, 0x50, 0x00, 0x43, 0x00,  0x31, 0x00, 0x04, 0x00, 0x0E, 0x00, 0x61, 0x00,  // A.N.P.C. 1.....a.
                0x6C, 0x00, 0x61, 0x00, 0x6E, 0x00, 0x70, 0x00,  0x63, 0x00, 0x31, 0x00, 0x03, 0x00, 0x0E, 0x00,  // l.a.n.p. c.1.....
                0x61, 0x00, 0x6C, 0x00, 0x61, 0x00, 0x6E, 0x00,  0x70, 0x00, 0x63, 0x00, 0x31, 0x00, 0x07, 0x00,  // a.l.a.n. p.c.1...
                0x08, 0x00, 0x48, 0x77, 0x80, 0x40, 0x1B, 0x2A,  0xCA, 0x01, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xF0,  // ..Hw?@.* .....
            };
            byte[] tmp = new byte[peerPackets.Length + followedBy.Length];
            Array.Copy(peerPackets, 0, tmp, 0, peerPackets.Length);
            Array.Copy(followedBy, 0, tmp, peerPackets.Length, followedBy.Length);
            return tmp;
        }

        byte[] GetTnapClientWillNegotiateAndAuthenticate()
        {
            byte[] expectedResponses = {
                IAC, WILL, OptionCode.AuthenticationOption,
                //----
#if false && !FOO
                0xFF, 0xFA, 0x25, 0x00, 0x0F, 0x00, 0x00, 0x28,  0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00,        // %....(.......
                //
                0x4E, 0x54, 0x4C, 0x4D, 0x53, 0x53, 0x50, 0x00,  0x01, 0x00, 0x00, 0x00, 0x97, 0x82, 0x08, 0xE2,  // NTLMSSP. ....??.
                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // ........ ........
                0x05, 0x01, 0x28, 0x0A, 0x00, 0x00, 0x00, 0x0F,                                                   // ..(.....
                //
                0xFF, 0xF0,                                                                                       // 
#else
                0xFF, 0xFA, 0x25, 0x00, 0x0F, 0x00, 0x00, 0x36,  0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00,        // %....(.......
                //
                0x4E, 0x54, 0x4C, 0x4D, 0x53, 0x53, 0x50, 0x00,  0x01, 0x00, 0x00, 0x00, 0x97, 0xB2, 0x08, 0xE2,
                0x07, 0x00, 0x07, 0x00, 0x2F, 0x00, 0x00, 0x00,  0x07, 0x00, 0x07, 0x00, 0x28, 0x00, 0x00, 0x00,
                0x05, 0x01, 0x28, 0x0A, 0x00, 0x00, 0x00, 0x0F,  0x41, 0x4C, 0x41, 0x4E, 0x50, 0x43, 0x31, 0x53,  //...ALANPC1S
                0x55, 0x49, 0x4C, 0x56, 0x45, 0x4E,                                                               //UILVEN
                //
                0xFF, 0xF0,                                                                                       // 
#endif
                //----
#if DREWOLD
                0xFF, 0xFA, 0x25, 0x00, 0x0F, 0x00, 0x02, 0xAC,  0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00,         // %.... .......
                //
                0x4E, 0x54, 0x4C, 0x4D, 0x53, 0x53, 0x50, 0x00,  0x03, 0x00, 0x00, 0x00, 0x18, 0x00, 0x18, 0x00,  // NTLMSSP. ........
                0x6C, 0x00, 0x00, 0x00, 0x18, 0x00, 0x18, 0x00,  0x84, 0x00, 0x00, 0x00, 0x0E, 0x00, 0x0E, 0x00,  // l....... ?.......
                0x48, 0x00, 0x00, 0x00, 0x08, 0x00, 0x08, 0x00,  0x56, 0x00, 0x00, 0x00, 0x0E, 0x00, 0x0E, 0x00,  // H....... V.......
                0x5E, 0x00, 0x00, 0x00, 0x10, 0x00, 0x10, 0x00,  0x9C, 0x00, 0x00, 0x00, 0x15, 0x82, 0x88, 0xE2,  // ^....... ?....??
                0x05, 0x01, 0x28, 0x0A, 0x00, 0x00, 0x00, 0x0F,  0x44, 0x00, 0x52, 0x00, 0x45, 0x00, 0x57, 0x00,  // ..(..... D.R.E.W.
                0x4F, 0x00, 0x4C, 0x00, 0x44, 0x00, 0x61, 0x00,  0x6C, 0x00, 0x61, 0x00, 0x6E, 0x00, 0x44, 0x00,  // O.L.D.a. l.a.n.D.
                0x52, 0x00, 0x45, 0x00, 0x57, 0x00, 0x4F, 0x00,  0x4C, 0x00, 0x44, 0x00, 0x1D, 0xEE, 0x38, 0x11,  // R.E.W.O. L.D..8.
                0x71, 0xCA, 0x53, 0x25, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // qS%.... ........
                0x00, 0x00, 0x00, 0x00, 0xB3, 0x6E, 0xC5, 0xCF,  0x09, 0xD3, 0x5A, 0x50, 0xEF, 0x4E, 0x73, 0x43,  // ....n .ZPNsC
                0x19, 0x00, 0xE8, 0xD4, 0x4E, 0x89, 0xDB, 0x0C,  0x0D, 0x86, 0xDC, 0xC6, 0x0F, 0x57, 0x0F, 0xF6,  // ..N?. .?.W.
                0xB1, 0x31, 0x5F, 0x96, 0x7B, 0xE4, 0xB7, 0xAB,  0x27, 0xEB, 0x0E, 0x31,                          // 1_?{䷫ '.1
                //
                0xFF, 0xF0,                                                                                       // 
#else
                0xFF, 0xFA, 0x25, 0x00, 0x0F, 0x00, 0x02, 0xAC,  0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00,         // %.... .......
                //
                0x4E, 0x54, 0x4C, 0x4D, 0x53, 0x53, 0x50, 0x00,  0x03, 0x00, 0x00, 0x00, /*LmChlgResp info*/0x18, 0x00, 0x18, 0x00,  // NTLMSSP. ........
                0x6C, 0x00, 0x00, 0x00, /*NtChlgResp info*/0x18, 0x00, 0x18, 0x00,  0x84, 0x00, 0x00, 0x00, 0x0E, 0x00, 0x0E, 0x00,  // l....... ?.......
                0x48, 0x00, 0x00, 0x00, 0x08, 0x00, 0x08, 0x00,  0x56, 0x00, 0x00, 0x00, 0x0E, 0x00, 0x0E, 0x00,  // H....... V.......
                0x5E, 0x00, 0x00, 0x00, 0x10, 0x00, 0x10, 0x00,  0x9C, 0x00, 0x00, 0x00, 0x15, 0x82, 0x88, 0xE2,  // ^....... ?....??
                0x05, 0x01, 0x28, 0x0A, 0x00, 0x00, 0x00, 0x0F,  0x41, 0x00, 0x4C, 0x00, 0x41, 0x00, 0x4E, 0x00,  // ..(..... D_R_E_W_
                0x50, 0x00, 0x43, 0x00, 0x31, 0x00, 0x61, 0x00,  0x6C, 0x00, 0x61, 0x00, 0x6E, 0x00, 0x41, 0x00,  // O_L_D_a. l.a.n.D.
                0x4C, 0x00, 0x41, 0x00, 0x4e, 0x00, 0x50, 0x00,  0x43, 0x00, 0x31, 0x00, /*LmChlgResp*/0x1D, 0xEE, 0x38, 0x11,  // R.E.W.O. L.D..8.
                0x71, 0xCA, 0x53, 0x25, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // qS%.... ........
                0x00, 0x00, 0x00, 0x00, /*NtChlgResp*/0xB3, 0x6E, 0xC5, 0xCF,  0x09, 0xD3, 0x5A, 0x50, 0xEF, 0x4E, 0x73, 0x43,  // ....n .ZPNsC
                0x19, 0x00, 0xE8, 0xD4, 0x4E, 0x89, 0xDB, 0x0C,  0x0D, 0x86, 0xDC, 0xC6, /*EncRndSessKey*/0x0F, 0x57, 0x0F, 0xF6,  // ..N?. .?.W.
                0xB1, 0x31, 0x5F, 0x96, 0x7B, 0xE4, 0xB7, 0xAB,  0x27, 0xEB, 0x0E, 0x31,                          // 1_?{䷫ '.1
                //
                0xFF, 0xF0,                                                                                       // 
#endif
            };
            return (byte[])expectedResponses.Clone();
        }

        [Test]
        public void Accepted()
        {
            m_doMunge = true;
            byte[] peerPackets = GetTnapServerDoThenChallenge(new byte[] {
                //---- Accept ----
                0xFF, 0xFA, 0x25, 0x02, 
                    0x0F, 0x00, 
                    NTLM_ACCEPT,
                    0xFF, 0xF0,
            });
            byte[] expectedResponses = GetTnapClientWillNegotiateAndAuthenticate();
            DoTest(peerPackets, empty, expectedResponses, null, new TelnetOption[] { TelnetOption.AuthenticationOption }, null);
        }

        [Test]
        public void Rejected()
        {
            m_doMunge = true;
            byte[] peerPackets = GetTnapServerDoThenChallenge(new byte[] {
                //---- Accept ----
                0xFF, 0xFA, 0x25, 0x02, 
                    0x0F, 0x00, 
                    NTLM_REJECT,
                    0xFF, 0xF0,
            });
            byte[] expectedResponses = GetTnapClientWillNegotiateAndAuthenticate();
            DoTest(peerPackets, empty, expectedResponses, null, new TelnetOption[] { TelnetOption.AuthenticationOption }, null);
        }

        [Test]
        public void RcvSendSubNeg_NoTnapOffered()
        {
            byte[] peerPackets = {
                IAC, DO, OptionCode.AuthenticationOption,
                //----
                IAC, SB, OptionCode.AuthenticationOption, 1,
                    0x99, 0x00,
                    0x98, 0x00,
                    0x01, 0x00,
                    0x02, 0x00,
                    IAC, SE,
            };
            byte[] expectedResponses = {
                IAC, WILL, OptionCode.AuthenticationOption,
                //----
                IAC, SB, OptionCode.AuthenticationOption, 0, 
                    0,0,
                    IAC, SE
            };
            //
            DoTest(peerPackets, empty, expectedResponses, null, new TelnetOption[] { TelnetOption.AuthenticationOption }, null);
        }

        [Test]
        public void RcvSendSubNeg_BadNullOfferList()
        {
            byte[] peerPackets = {
                IAC, DO, OptionCode.AuthenticationOption,
                //----
                IAC, SB, OptionCode.AuthenticationOption, 1,
                    IAC, SE,
            };
            byte[] expectedResponses = {
                IAC, WILL, OptionCode.AuthenticationOption,
                //----
                IAC, SB, OptionCode.AuthenticationOption, 0, 
                    0,0,
                    IAC, SE
            };
            //
            DoTest(peerPackets, empty, expectedResponses, null, new TelnetOption[] { TelnetOption.AuthenticationOption }, null);
        }

        [Test]
        public void RcvAuthSubNeg_BadShort_ieNoCommandByte()
        {
            byte[] peerPackets = {
                IAC, DO, OptionCode.AuthenticationOption,
                //----
                IAC, SB, OptionCode.AuthenticationOption,
                    // !!!Truncated before the command-code!!! -- 1,
                    IAC, SE,
            };
            byte[] expectedResponses = {
                IAC, WILL, OptionCode.AuthenticationOption,
                //----  
                IAC, SB, OptionCode.AuthenticationOption, 0, 
                    0,0,
                    IAC, SE
            };
            //
            try {
                DoTest(peerPackets, empty, expectedResponses, null, new TelnetOption[] { TelnetOption.AuthenticationOption }, null);
                Assert.Fail("should have thrown!");
            } catch {
            }
        }

        class FailPromptAuthenticationNtlmTelnetHandler : AuthenticationNtlmTelnetHandler
        {
            public FailPromptAuthenticationNtlmTelnetHandler(bool? promptOrForce)
                : base(promptOrForce)
            {
            }

            protected override bool Prompt(string message)
            {
                throw new NotImplementedException("TEST should not be called!");
            }
        }

    }
}
